unit D3D_PRender;

interface
uses
 Render,J_Level, Windows, OLE2, SysUtils, Forms, Classes,
 D3DRMObj, D3DTypes, D3DRMDef, DDraw, D3D, D3Drm, dxtools,
 D3DrmWin, D3Dcaps, Graph_files, Files, FileOperations;

Type

TPreviewRender=class
 CamX,CamY,CamZ:Double;
 PCH,YAW:Double;
 Viewer:TForm;
 Constructor Create(aForm:TForm);
 Procedure Initialize;virtual;abstract;
 Procedure ClearSectors;virtual;abstract;
 Procedure AddSector(s:TJKSector);virtual;abstract;
 Procedure InsertSector(n:integer;s:TJKSector);virtual;abstract;
 Procedure DeleteSector(n:integer);virtual;abstract;
 Procedure UpdateSector(n:integer;s:TJKSector);virtual;abstract;
 Procedure SetViewPort(x,y,w,h:integer);virtual;abstract;
 Procedure Redraw;virtual;abstract;
end;

TD3DTexture=class
 ITexture:IDirect3DRMTexture;
 imh:D3DRMIMAGE;
 Constructor CreateFromMat(const Mat:string;Pal:TCMPPal;d3drm:IDirect3DRM);
 Destructor Destroy;override;
end;


TD3DRenderer=class(TPreviewRender)
   DDBD: dword;
   FBPP:Dword;

   FDriverIndex: Integer;
   FDriverGUID: PGUID;

   FD3DRM:     IDirect3DRM;
   FClip:      IDirectDrawClipper;
   FDev:       IDirect3DRMDevice;
   FWDev:      IDirect3DRMWinDevice;

   FCamera:    IDirect3DRMFrame;
   FScene:     IDirect3DRMFrame;
   FView:      IDirect3DRMViewport;
   FSubScene:  IDirect3DRMFrame;
   nomore:boolean;
   group:array[0..1000] of dword;
   TXList,CmpList:TStringList;

 Constructor Create(aForm:TForm);
 Procedure Initialize;override;
 Procedure ClearSectors;override;
 Procedure AddSector(s:TJKSector);override;
 Procedure InsertSector(n:integer;s:TJKSector);override;
 Procedure DeleteSector(n:integer);override;
 Procedure UpdateSector(n:integer;s:TJKSector);override;
 Procedure SetViewPort(x,y,w,h:integer);override;
 Procedure Redraw;override;
 Destructor Destroy;override;
Private
 Function SecToMesh(s:TJKSector):IDirect3DRMMesh;
 Procedure InitializeDevice;
 Procedure SetDriver(n:integer);
 procedure RebuildDevice(bForce: Boolean);
 Procedure ClearTXList;
 Function LoadD3DTexture(const name,cmp:string):TD3DTexture;
end;

implementation

Constructor TPreviewRender.Create(aForm:TForm);
begin
 Viewer:=aForm;
end;

Constructor TD3DTexture.CreateFromMat(const Mat:string;Pal:TCMPPal;d3drm:IDirect3DRM);
var 
    i:integer;
    pe:^D3DRMPALETTEENTRY;
    pb:pchar;
    mf:TMat;
    f:TFile;
begin
 f:=OpenGameFile(mat);
 mf:=TMat.Create(f);

 With Imh do
 begin
  width:=mf.info.width;
  height:=mf.info.Height;
  bytes_per_line:=width;
  aspectx:=1;
  aspecty:=1;
  depth:=8;
  rgb:=0;
  GetMem(pb,width*height);
  buffer1:=pb;

  for i:=0 to height-1 do
  begin
   mf.GetLine((pb+i*width)^);
  end;

  buffer2:=nil;
  red_mask:=$ff;
  green_mask:=$ff;
  blue_mask:=$ff;
  alpha_mask:=0;
  palette_size:=256;
  GetMem(Palette,sizeof(D3DRMPALETTEENTRY)*256);
  for i:=0 to 255 do
  With Pal[i] do
  begin
   pe:=Pointer(Pchar(Palette)+i*sizeof(D3DRMPALETTEENTRY));
   pe^.red:=r;
   pe^.green:=g;
   pe^.blue:=b;
   pe^.flags:=0;{D3DRMPALETTE_FREE}
  end;
 end;
 mf.free;
 DXCheck(d3drm.CreateTexture(imh,ITexture));
end;

Destructor TD3DTexture.Destroy;
begin
 FreeMem(imh.buffer1);
 FreeMem(imh.palette);
end;


 type
  TD3DDriverInfo = record
    id:                TGUID;
    DeviceDescription: string;
    DeviceName:        string;
    HWDeviceDesc:      D3DDeviceDesc ;
    HELDeviceDesc:     D3DDeviceDesc ;
  end;

const
  MAXDRIVERS = 8;

var
  {a few global variables storing driver info}
  _D3Ddrivers: array[0.. MAXDRIVERS-1] of TD3DDriverInfo;
  _D3DdriverCount: Integer;
  _bDriversInitialized: Boolean;

function _EnumCallBack(lpGuid: PGUID ;
      lpDeviceDescription: LPSTR ; lpDeviceName: LPSTR ;
      const lpD3DHWDeviceDesc: D3DDeviceDesc ;
      const lpD3DHELDeviceDesc: D3DDeviceDesc ;
      lpUserArg: Pointer ): HRESULT ; stdcall ;
var
  dev: ^D3DDeviceDesc;
  DDBD: DWORD;
begin
  dev := @lpD3DHWDeviceDesc;
  DDBD := DWORD(lpUserArg);
  if Integer(lpD3DHWDeviceDesc.dcmColorModel) = 0 then dev := @lpD3DHELDeviceDesc;
  if (dev^.dwDeviceRenderBitDepth and DDBD) <> 0 then
  begin
    {current bit depth is supported by this driver}
    with _D3DDrivers[_D3DdriverCount] do
    begin
      Move(lpGUID^, id, SizeOf(TGUID));
      Move(lpD3DHWDeviceDesc, HWDeviceDesc, SizeOf(D3DDeviceDesc));
      Move(lpD3DHELDeviceDesc, HELDeviceDesc, SizeOf(D3DDeviceDesc));
      DeviceDescription := StrPas(lpDeviceDescription);
      DeviceName := StrPas(lpDeviceName);
    end;
    Inc(_D3DdriverCount)
  end;
  Result := D3DENUMRET_OK;
  if _D3DDriverCount >= MAXDRIVERS then Result := D3DENUMRET_CANCEL;
end;

procedure _InitializeDrivers(Renderer: TD3DRenderer);
var
  d3d: IDirect3D;
  dd: IDirectDraw;
begin
  if not _bDriversInitialized then
  begin
    DXCheck(DirectDrawCreate(nil, dd, nil));
    try
      DXCheck(dd.QueryInterface(IID_IDirect3D, d3d));
      try
        _bDriversInitialized := True;
        DXCheck(d3d.EnumDevices(_EnumCallback, Pointer(Renderer.DDBD)));
      finally
        COMRelease(IUnknown(d3d));
      end;
    finally
      COMRelease(IUnknown(dd));
    end;
  end;
end;

Constructor TD3DRenderer.Create(aForm:TForm);
begin
 Inherited Create(aForm);
 TxList:=TStringList.Create;
 TXList.Sorted:=true;
 CmpList:=TStringList.Create;
 CmpList.Sorted:=true;
end;

Procedure TD3DRenderer.Initialize;
var
  lightframe: IDirect3DRMFrame;
  light1, light2: IDirect3DRMLight;
  dc: HDC;
begin
 {retrieve bits per pixel}
 DXCheck(Direct3DRMCreate(FD3DRM));

 DDBD:=8+16+24;
 dc := GetDC(Viewer.Handle);
 FBPP := GetDeviceCaps(dc, BITSPIXEL);
 ReleaseDC(Viewer.Handle, dc);
 _InitializeDrivers(Self);

 {create scene}
 DXCheck(DirectDrawCreateClipper(0, FClip, nil));
 DXCheck(FClip.SetHWnd(0, Viewer.Handle));
 DXCheck(FD3DRM.CreateFrame(nil, FScene));
 DXCheck(FD3DRM.CreateFrame(FScene, FCamera));
 DXCheck(FCamera.SetPosition(FScene, 0.0, 0.0, 0.0));
(* DXCheck(FD3DRM.CreateFrame(FScene, lightframe));
 try
   {create lights}
{   DXCheck(FD3DRM.CreateLightRGB(D3DRMLIGHT_DIRECTIONAL, 0.9, 0.9, 0.9, light1));
   try
     DXCheck(lightframe.AddLight(light1));
   finally
     COMRelease(IUnknown(light1));
   end;}
  { DXCheck(FD3DRM.CreateLightRGB(D3DRMLIGHT_AMBIENT, 1, 1, 1, light2));
   try
     DXCheck(lightframe.AddLight(light2));
   finally
     COMRelease(IUnknown(light2));
   end;}
   DXCheck(lightframe.SetPosition(FScene, 2.0, 0.0, 22.0));
 finally
   COMRelease(IUnknown(lightframe));
 end; *)

 {create 'world' frame that will contain the user-defined objects}
 DXCheck(FD3DRM.CreateFrame(FScene, FSubScene));

 {set positions and orientations of frames}
 DXCheck(FCamera.SetPosition(FScene, 0.0, 0.0, -15.0));
 DXCheck(FCamera.SetOrientation(FScene, 0.0, 0.0, 1.0, 0.0, 1.0, 0.0));
      
 DXCheck(FSubScene.SetPosition(FScene, 0.0, 0.0, 0.0));
 DXCheck(FSubScene.SetOrientation(FScene, 0.0, 0.0, 1.0, 0.0, 1.0, 0.0));

 {create device and vieport}
 FDriverGUID:=nil;
 InitializeDevice;
 FView.SetProjection(D3DRMPROJECT_PERSPECTIVE);
 FSubScene.SetZBufferMode(D3DRMZBUFFER_ENABLE);


{ FInitialized := True;}

 {user callback for getting content of scene}

end;

procedure TD3DRenderer.InitializeDevice;
var
  w, h: Integer;
  r : HResult;
begin
  DXCheck(FD3DRM.CreateDeviceFromClipper(FClip, FDriverGUID, Viewer.ClientWidth, Viewer.ClientHeight, FDev));
  DXCheck(FDev.QueryInterface(IID_IDirect3DRMWinDevice, FWDev));
  w := FDev.GetWidth;
  h := FDev.GetHeight;
  if COMFailed(r, FD3DRM.CreateViewport(FDev, FCamera, 0, 0, w, h, FView)) then
  begin
    COMRelease(IUnknown(FWDev));
    COMRelease(IUnknown(FDev));
    DXCheck(r);
  end;
  if COMFailed(r, FView.SetBack(5000.0)) then
  begin
    COMRelease(IUnknown(FView));
    COMRelease(IUnknown(FWDev));
    COMRelease(IUnknown(FDev));
    DXCheck(r);
  end;

  {set render quality}
  DXCheck(FDev.SetQuality(D3DRMFILL_SOLID or D3DRMSHADE_GOURAUD));
  DXCheck(FDev.SetTextureQuality(D3DRMTEXTURE_LINEAR));

  FDev.GetQUality;

  {render quality part 2 - standard incantation}
  if FBPP = 1 then
  begin
    DXCheck(FDev.SetShades(4));
    DXCheck(FD3DRM.SetDefaultTextureShades(4));
  end
  else if FBPP = 16 then
  begin
    DXCheck(FDev.SetShades(32));
    DXCheck(FD3DRM.SetDefaultTextureColors(64));
    DXCheck(FD3DRM.SetDefaultTextureShades(32));
  end
  else
  if (FBPP = 24) or (FBPP = 32) then
  begin
    DXCheck(FDev.SetShades(256));
    DXCheck(FD3DRM.SetDefaultTextureColors(64));
    DXCheck(FD3DRM.SetDefaultTextureShades(256));
  end;

end;


Procedure TD3DRenderer.Redraw;
var vs:IDirect3DRMVisualArray;
begin
{ FSubScene.GetVisuals(vs);
 for i:=vs.GetSize-1 downto 0 do
 begin
  vs.getElement(i,v);
  FSubScene.DeleteVisual(v);
 end;
end;}
 FSubScene.GetVisuals(vs);
 DXCheck(FView.Clear);

 DXCheck(FCamera.AddTranslation(D3DRMCOMBINE_REPLACE ,CamX,CamY,CamZ));
 {FCamera.AddScale(D3DRMCOMBINE_BEFORE,scale,scale,scale);}
 FCamera.AddRotation(D3DRMCOMBINE_AFTER,1,0,0,PCH/180*PI);
 FCamera.AddRotation(D3DRMCOMBINE_AFTER,0,1,0,YAW/180*PI);
 
 DXCheck(FView.Render(FScene));
 FDev.Update;
end;

Procedure TD3DRenderer.SetViewPort(x,y,w,h:integer);
begin
end;

Function TD3DRenderer.SecToMesh(s:TJKSector):IDirect3DRMMesh;
const
     maxvx=24;
var i,j:integer;
    Mesh:IDirect3DRMMesh;
    Vxs:array[0..maxvx-1] of D3DRMVertex;
    cvx:^D3DRMVertex;
    nvx:integer;
    id:D3DRMGROUPINDEX;
    ttx:TD3DTexture;
    pim:LPD3DRMIMAGE;

begin
 DXCheck(FD3DRM.CreateMesh(Mesh));

 For i:=0 to maxvx-1 do group[i]:=i;

 for i:=0 to s.surfaces.count-1 do
 With s.surfaces[i] do
 begin
  if Material='' then continue;

 DXCHeck(Mesh.AddGroup(Vertices.Count,1,Vertices.Count,
  group[0],ID));

 ttx:=LoadD3DTexture(Material,s.ColorMap);

 DXCheck(Mesh.SetGroupTexture(ID,ttx.Itexture));

 pim:=ttx.Itexture.GetImage;
 if pim=nil then;

 nvx:=Vertices.Count;
 FillChar(vxs,sizeof(vxs),0);
 if nvx>24 then nvx:=24;

 for j:=0 to nvx-1 do
 With Vertices[j],TXVertices[j] do
 begin
  cvx:=@vxs[j];
  CVX^.position.x:=x;
  CVX^.position.y:=y;
  CVX^.position.z:=z;
  CVX^.normal.x:=normal.dx;
  CVX^.normal.y:=normal.dy;
  CVX^.normal.z:=normal.dz;
  CVX^.Color:=Round(Intensity*255);
  cvx^.tu:=u/ttx.imh.width;
  cvx^.tv:=v/ttx.imh.height;
 end;
  Mesh.SetVertices(ID,0,nvx,vxs[0]);
  Mesh.SetGroupMapping(ID,D3DRMMAP_PERSPCORRECT{+D3DRMMAP_WRAPU+D3DRMMAP_WRAPV});
  Mesh.SetGroupQuality(ID,D3DRMFILL_SOLID or D3DRMSHADE_GOURAUD);
 { Mesh.SetGroupColorRGB(ID,1,1,1);}
 end;
Result:=mesh;
end;

procedure TD3DRenderer.RebuildDevice(bForce: Boolean);
var
  oldDither: BOOL;
  oldQuality: D3DRMRENDERQUALITY;
  oldShades: DWORD;
  w, h, w0, h0: Integer;
  r: HRESULT;
begin
  w0 := Viewer.ClientWidth;
  h0 := Viewer.ClientHeight;
  if Assigned(FDev) and ((FDev.GetWidth<>w0) or (FDev.GetHeight<>h0) or bForce) then
  begin
    oldDither := FDev.GetDither;
    oldQuality := FDev.GetQuality;
    oldShades := FDev.GetShades;

    COMRelease(IUnknown(FView));
    COMRelease(IUnknown(FWDev));
    COMRelease(IUnknown(FDev));

    DXCheck(FD3DRM.CreateDeviceFromClipper(FClip, FDriverGUID, w0, h0, FDev));
    DXCheck(FDev.QueryInterface(IID_IDirect3DRMWinDevice, FWDev));
    w := FDev.GetWidth;
    h := FDev.GetHeight;
    if COMFailed(r, FD3DRM.CreateViewport(FDev, FCamera, 0, 0, w, h, FView)) then
    begin
      COMRelease(IUnknown(FWDev));
      COMRelease(IUnknown(FDev));
      DXCheck(r);
    end;
    if COMFailed(r, FView.SetBack(5000.0)) then
    begin
      COMRelease(IUnknown(FView));
      COMRelease(IUnknown(FWDev));
      COMRelease(IUnknown(FDev));
      DXCheck(r);
    end;
    DXCheck(FDev.SetQuality(oldQuality));
    DXCheck(FDev.SetDither(oldDither));
    DXCheck(FDev.SetShades(oldShades));
  end;
end;


procedure TD3DRenderer.SetDriver(n: Integer);
var
  pid: PGUID;
begin
  if (n>=0) and (n<_D3DdriverCount) then
  begin
    pid := @_D3Ddrivers[n].id;
    FDriverIndex := n;
  end
  else
  begin
    {default driver}
    pid := nil;
    FDriverIndex := -1;
  end;
  FDriverGUID := pid;
  RebuildDevice(True);
end;


Destructor TD3DRenderer.Destroy;
begin
    if Assigned(FView) then COMRelease(IUnknown(FView));
    if Assigned(FWDev) then COMRelease(IUnknown(FWDev));
    if Assigned(FDev) then COMRelease(IUnknown(FDev));
    COMRelease(IUnknown(FSubScene));
    COMRelease(IUnknown(FCamera));
    COMRelease(IUnknown(FScene));
    COMRelease(IUnknown(FClip));
  COMRelease(IUnknown(FD3DRM));
  ClearTXList;
  TXList.Free;
  CmpList.Free;
  inherited Destroy;
end;

Procedure TD3DRenderer.ClearSectors;
begin
end;

Procedure TD3DRenderer.AddSector(s:TJKSector);
begin
 DXCheck(FSubScene.AddVisual(SecToMesh(s)));
end;

Procedure TD3DRenderer.InsertSector(n:integer;s:TJKSector);
begin
end;

Procedure TD3DRenderer.DeleteSector(n:integer);
begin
end;

Procedure TD3DRenderer.UpdateSector(n:integer;s:TJKSector);
begin
end;

Procedure TD3DRenderer.ClearTXList;
var i:integer;
begin
 for i:=0 to TXList.Count-1 do
  TXList.Objects[i].Free;
 TXList.Clear;
 for i:=0 to CMPList.Count-1 do
  FreeMem(Pointer(CMPList.Objects[i]));
  CmpList.Clear;
end;

Function TD3DRenderer.LoadD3DTexture(const name,cmp:string):TD3DTexture;
var i:integer;
    Ttx:TD3DTexture;
    pcmp:^TCMPPal;
    f:TFile;
begin
 i:=TXlist.IndexOf(name);
 if i<>-1 then
 begin
  Result:=TD3DTexture(TXList.Objects[i]);
  exit;
 end;
 i:=CmpList.IndexOf(cmp);
 if i<>-1 then pcmp:=Pointer(CmpList.Objects[i]) else
 begin
  GetMem(pcmp,sizeof(pcmp^));
  f:=OpenGameFile(cmp);
  LoadCmp(f,pcmp^);
  f.Fclose;
  CmpList.AddObject(cmp,TObject(pcmp));
 end;
 ttx:=TD3DTexture.CreateFromMat(name,pcmp^,Fd3drm);
 TXList.AddObject(name,ttx);
 Result:=ttx;
end;

end.
