unit u_3dos;

interface
uses geometry, SysUtils, misc_utils, GlobalVars;

Type

T3DOFace=class(TPolygon)
 imat:integer;
 ftype:Longint;
 geo,light,tex:integer;
 extra_l:single;
end;

T3DOFaces=class(TPolygons)
 Function GetItem(n:integer):T3DOface;
 Procedure SetItem(n:integer;v:T3DOface);
Property Items[n:integer]:T3DOface read GetItem write SetItem; default;
end;

T3DOMesh=class
 name:string;
 faces:T3DOFaces;
 COnstructor Create;
 Function GetVXs:TVertices;
 Property vertices:Tvertices read GetVXs;
 Destructor Destroy;override;
 Function FindRadius:double;
end;

T3DOMeshes=class(TList)
 Function GetItem(n:integer):T3DOMesh;
 Procedure SetItem(n:integer;v:T3DOMesh);
Property Items[n:integer]:T3DOMesh read GetItem write SetItem; default;
end;


T3DO=class
 ucount:integer;
 Mats:TStringList;
 Meshes:T3DOMeshes;
 Constructor CreateNew;
 Constructor CreateFrom3DO(const name:string;lod:integer);
 Destructor Destroy;override;
 Function NewMesh:T3DOMesh;
 Function GetMat(n:integer):string;
 Procedure GetBBox(var box:TThingBox);
 Procedure SaveToFile(const name:string);
 Function FindRadius:double;
end;

Function Load3DO(const name:string):T3DO;
Procedure Free3DO(var a3DO:T3DO);
{These function must be use to load and free
 3DOs, not T3DO.CreateFrom3DO, T3Do.Free}

implementation

uses files, FileOperations, lev_utils;

var
  L3DOs:TStringList;


Function Load3DO(const name:string):T3DO;
var i:integer;
begin
 i:=L3DOs.IndexOf(name);
 if i<>-1 then begin Result:=T3DO(L3DOs.Objects[i]); inc(Result.ucount); end
 else
 begin
  try
   Result:=T3DO.CreateFrom3DO(name,3);
   L3DOs.AddObject(name,Result);
   Result.ucount:=1;
  except
   on Exception do begin result:=nil; PanMessageFmt(mt_warning,'Cannot load %s',[name]); end;
  end;
 end;
end;

Procedure Free3DO(var a3DO:T3DO);
var i:integer;
begin
 if a3DO=nil then exit;
try
try
 Dec(a3DO.ucount);
 if A3Do.ucount<=0 then
 begin
   i:=L3DOs.IndexOfObject(a3DO);
   if i<>-1 then L3DOs.Delete(i);
  a3DO.Destroy;
 end;
finally
 a3DO:=nil;
end;
except
 On Exception do;
end;
end;



Function T3DOFaces.GetItem(n:integer):T3DOFace;
begin
 if (n<0) or (n>=count) then raise EListError.CreateFmt('3DO Face Index is out of bounds: %d',[n]);
 Result:=T3DOFace(List[n]);
end;

Procedure T3DOFaces.SetItem(n:integer;v:T3DOFace);
begin
 if (n<0) or (n>=count) then raise EListError.CreateFmt('3DO Face Index is out of bounds: %d',[n]);
 List[n]:=v;
end;

Function T3DOMeshes.GetItem(n:integer):T3DOMesh;
begin
 if (n<0) or (n>=count) then raise EListError.CreateFmt('3DO Mesh Index is out of bounds: %d',[n]);
 Result:=T3DOMesh(List[n]);
end;

Procedure T3DOMeshes.SetItem(n:integer;v:T3DOMesh);
begin
 if (n<0) or (n>=count) then raise EListError.CreateFmt('3DO Mesh Index is out of bounds: %d',[n]);
 List[n]:=v;
end;


Constructor T3DO.CreateNew;
begin
 Mats:=TStringList.create;
 Meshes:=T3DOMeshes.Create;
end;

Destructor T3DO.Destroy;
var i,j:integer;
begin
try
 for i:=0 to Meshes.count-1 do
 With Meshes[i] do
 begin
  For j:=0 to Vertices.count-1 do Vertices[j].free;
  For j:=0 to Faces.count-1 do faces[j].free;
  free;
 end;
finally
 if i=0 then;
 if j<>0 then;
end;
 Inherited destroy;
end;

Function T3DO.NewMesh:T3DOMesh;
begin
 Result:=T3DOMesh.Create;
end;

Function T3DO.GetMat(n:integer):string;
begin
 if (n<0) or (n>=mats.count) then result:=''
 else result:=Mats[n];
end;

Procedure T3DO.GetBBox(var box:TThingBox);
var i,j:integer;
    x1,x2,y1,y2,z1,z2:double;
begin
 x1:=99999;
 x2:=-99999;
 y1:=99999;
 y2:=-99999;
 z1:=99999;
 z2:=-99999;

 for i:=0 to Meshes.count-1 do
 With Meshes[i] do
 for j:=0 to Vertices.count-1 do
 With Vertices[j] do
 begin
  if x<x1 then x1:=x;
  if x>x2 then x2:=x;
  if y<y1 then y1:=y;
  if y>y2 then y2:=y;
  if z<z1 then z1:=z;
  if z>z2 then z2:=z;
 end;
 if x1=99999 then FillChar(Box,sizeof(Box),0)
 else
 begin
  box.x1:=x1;
  box.x2:=x2;
  box.y1:=y1;
  box.y2:=y2;
  box.z1:=z1;
  box.z2:=z2;
 end;
end;

COnstructor T3DOMesh.Create;
begin
 Faces:=T3DOFaces.Create;
 Faces.VXList:=TVertices.Create;
end;

Function T3DOMesh.GetVXs:TVertices;
begin
 Result:=faces.VXList;
end;

Destructor T3DOMesh.Destroy;
begin
 Faces.VXList.Destroy;
 Faces.Destroy;
end;

Function T3DOMesh.FindRadius:double;
var i:integer;
    crad:double;
begin
 Result:=0;
 for i:=0 to Vertices.count-1 do
 With Vertices[i] do
 begin
  crad:=Sqrt(sqr(x)+sqr(y)+sqr(z));
  if crad>result then Result:=crad;
 end;
end;


Function T3DO.FindRadius:double;
var i,j:integer;
    crad:double;
begin
 Result:=0;
 for i:=0 to Meshes.count-1 do
 begin
  crad:=Meshes[i].FindRadius;
  if crad>result then Result:=crad;
 end;
end;

{$i 3do_io.inc}


Initialization
begin
 L3DOs:=TStringList.Create;
 L3DOs.Sorted:=true;
end;

end.
