unit u_undo;

interface

uses J_Level, SysUtils;

{$A-}

Type
 PWord=^Word;
 PInt=^SmallInt;

 TVXRec=record
  x,y,z:double;
 end;
 PVXRec=^TVXRec;

 TSFVX=record
  nvx:word;
  u,v:single;
  intensity:single;
  r,g,b:single;
 end;

 PSFVX=^TSFVX;

 TSecRec=record
  Flags:longint;
  Ambient:single;
  extra:single;
  ColorMap:array[0..31] of char;
  Tint:record r,g,b:single end;
  Sound:array[0..47] of char;
  snd_vol:single;
  layer:SmallInt;
  nvxs,nsfs:integer;
 end;

 TSurfRec=record
  AdjSC,AdjSF:SmallInt;
  AdjoinFlags:Longint;
  Material:array[0..47] of char;
  SurfFlags,FaceFlags:Longint;
  geo,light,tex:byte;
  extraLight:single;
  uscale,vscale:single;
  nvxs:integer;
 end;

TLightRec=record
 X,Y,Z:double;
 Intensity:double;
 Range:double;
 flags:longint;
 R,G,B:Single;
 rgbintensity:single;
 layer:SmallInt;
end;

TThingRec=record
 Sec:SmallInt;
 name:array[0..47] of char;
 X,Y,Z:double;
 PCH,YAW,ROL:Double;
 layer:SmallInt;
 vals:array[0..255] of char;
end;


TCogRec=record
 name:array[0..47] of char;
 {vals:TCOGValues;}
end;

Procedure StartUndoRec(name:string);
Procedure AddSecUndo(sec:TJKSector;how:integer);
Procedure AddThingUndo(th:TJKThing);
Procedure AddLightUndo(lt:TJedLight);
Procedure EndUndoRec;
Function GetRecName:string;
Procedure ApplyUndo;

Procedure GetLight(light:TJedLight;var lrec:TLightRec);
Procedure SetLight(light:TJedLight;var lrec:TLightRec);
Function GetLightRecSize:integer;

Function GetThingRecSize:integer;
Procedure GetThing(thing:TJKThing;var trec:TThingRec);
Procedure SetThing(thing:TJKThing;var trec:TThingRec);

Function GetSecRecSize:Integer;
Procedure GetSec(sec:TJKSector;var srec:TSecRec);
Procedure SetSec(sec:TJKSector;var srec:TSecRec);
Procedure GetSurf(surf:TJKSurface;var sfrec:TSurfRec);
Function GetSurfRecSize:Integer;
Procedure SetSurf(surf:TJKSurface;var sfrec:TSurfRec);
Function SCVertSize(vs:TJKVertices):integer;
Procedure GetSCVertices(vs:TJKVertices;vxrecs:PVXRec);
Procedure SetSCVertices(vs:TJKVertices;nvxs:integer;vxrecs:PVXRec);
Function SFVertSize(vs:TJKVertices):integer;
Procedure GetSFVertices(surf:TJKSurface;pvxs:PSFVX);
Procedure SetSFVertices(sec:TJKSector;surf:TJKSUrface;nvxs:integer;pvxs:PSFVX);


implementation

uses Misc_utils;

Function GetSecRecSize:Integer;
begin
 result:=sizeof(TSecRec);
end;

Procedure GetSec(sec:TJKSector;var srec:TSecRec);
begin
 with sec do
 begin
  srec.flags:=Flags;
  srec.Ambient:=Ambient;
  srec.extra:=extra;
  StrLCopy(srec.ColorMap,pchar(Colormap),Sizeof(srec.colormap));
  srec.tint.r:=Tint.dx;
  srec.tint.g:=Tint.dy;
  srec.tint.b:=Tint.dz;
  StrLCopy(srec.Sound,Pchar(Sound),sizeof(srec.Sound));
  srec.snd_vol:=snd_vol;
  srec.layer:=Layer;
  srec.nvxs:=Vertices.count;
  srec.nsfs:=Surfaces.count;
 end;
end;

Procedure SetSec(sec:TJKSector;var srec:TSecRec);
begin
 with sec do
 begin
  Flags:=srec.flags;
  Ambient:=srec.Ambient;
  extra:=srec.extra;
  ColorMap:=srec.Colormap;
  Tint.dx:=srec.tint.r;
  Tint.dy:=srec.tint.g;
  Tint.dz:=srec.tint.b;
  sound:=Srec.Sound;
  snd_vol:=srec.snd_vol;
  layer:=srec.layer;
 end;
end;

Procedure GetSurf(surf:TJKSurface;var sfrec:TSurfRec);
begin
With surf do
begin
 if adjoin=nil then sfrec.adjSC:=-1 else
 begin
  sfrec.AdjSC:=adjoin.Sector.num;
  sfrec.adjSF:=adjoin.Num;
 end;
 sfrec.AdjoinFlags:=AdjoinFlags;
 StrLCopy(sfrec.Material,pchar(Material),sizeof(sfrec.Material));
 sfrec.SurfFlags:=SurfFlags;
 sfrec.FaceFlags:=FaceFlags;
 sfrec.geo:=geo;
 sfrec.light:=light;
 sfrec.tex:=tex;
 sfrec.extraLight:=extralight;
 sfrec.uscale:=uscale;
 sfrec.vscale:=vscale;
 sfrec.nvxs:=Vertices.count;
end;
end;

Function GetSurfRecSize:Integer;
begin
 Result:=sizeof(TSurfRec);
end;

Procedure SetSurf(surf:TJKSurface;var sfrec:TSurfRec);
begin
 surf.mark:=sfrec.AdjSC*65536+sfrec.AdjSF;
 surf.AdjoinFlags:=sfrec.AdjoinFlags;
 surf.Material:=sfrec.Material;
 surf.SurfFlags:=sfrec.SurfFlags;
 surf.FaceFlags:=sfrec.FaceFlags;
 surf.geo:=sfrec.geo;
 surf.light:=sfrec.light;
 surf.tex:=sfrec.tex;
 surf.extralight:=sfrec.extraLight;
 surf.uscale:=sfrec.uscale;
 surf.vscale:=sfrec.vscale;
end;

Function SCVertSize(vs:TJKVertices):integer;
begin
 result:=vs.count*sizeof(TVXRec);
end;

Procedure GetSCVertices(vs:TJKVertices;vxrecs:PVXRec);
var i:integer;
begin
 for i:=0 to vs.count-1 do
 with vs[i] do
 begin
  vxrecs^.x:=x;
  vxrecs^.y:=y;
  vxrecs^.z:=z;
  inc(vxrecs);
 end;
end;

Procedure SetSCVertices(vs:TJKVertices;nvxs:integer;vxrecs:PVXRec);
var i:integer;
begin
 for i:=0 to nvxs-1 do
 with vs[i] do
 begin
  x:=vxrecs^.x;
  y:=vxrecs^.y;
  z:=vxrecs^.z;
  inc(vxrecs);
 end;
end;


Function SFVertSize(vs:TJKVertices):integer;
begin
 result:=vs.count*sizeof(TSFVX);
end;

Procedure GetSFVertices(surf:TJKSurface;pvxs:PSFVX);
var i:integer;
begin
 for i:=0 to surf.vertices.count-1 do
 with surf.vertices[i],surf.txvertices[i] do
 begin
  pvxs^.nvx:=surf.vertices[i].num;
  pvxs^.u:=u;
  pvxs^.v:=v;
  pvxs^.intensity:=Intensity;
  pvxs^.r:=r;
  pvxs^.g:=g;
  pvxs^.b:=b;
  inc(pvxs);
 end;
end;

Procedure SetSFVertices(sec:TJKSector;surf:TJKSUrface;nvxs:integer;pvxs:PSFVX);
var i:integer;
begin
 for i:=0 to nvxs-1 do
 with surf.txvertices[i] do
 begin
  surf.vertices[i]:=sec.Vertices[pvxs^.nvx];
  u:=pvxs^.u;
  v:=pvxs^.v;
  Intensity:=pvxs^.intensity;
  r:=pvxs^.r;
  g:=pvxs^.g;
  b:=pvxs^.b;
  inc(pvxs);
 end;
end;

Function GetLightRecSize:integer;
begin
 Result:=Sizeof(TLightRec);
end;

Procedure GetLight(light:TJedLight;var lrec:TLightRec);
begin
 With light do
 begin
  lrec.X:=x;
  lrec.Y:=y;
  lrec.Z:=z;
  lrec.Intensity:=Intensity;
  lrec.Range:=range;
  lrec.flags:=flags;
  lrec.R:=r;
  lrec.G:=g;
  lrec.B:=b;
  lrec.rgbintensity:=rgbintensity;
  lrec.layer:=layer;
 end;
end;

Procedure SetLight(light:TJedLight;var lrec:TLightRec);
begin
 With light do
 begin
  x:=lrec.X;
  y:=lrec.Y;
  z:=lrec.Z;
  Intensity:=lrec.Intensity;
  range:=lrec.Range;
  flags:=lrec.flags;
  r:=lrec.R;
  g:=lrec.G;
  b:=lrec.B;
  rgbintensity:=lrec.rgbintensity;
  layer:=lrec.layer;
 end;
end;

Function GetThingRecSize:integer;
begin
 Result:=Sizeof(TThingRec);
end;

Procedure GetThing(thing:TJKThing;var trec:TThingRec);
var st:string;
    i:integer;
begin
 With thing do
 begin
  if sec=nil then trec.Sec:=-1 else trec.sec:=Sec.num;
  StrLCopy(trec.Name,pchar(Name),sizeof(trec.Name));
  trec.X:=X;
  trec.Y:=Y;
  trec.Z:=Z;
  trec.PCH:=PCH;
  trec.YAW:=YAW;
  trec.ROL:=ROL;
  trec.layer:=layer;
  st:='';
  for i:=0 to vals.count-1 do
  With vals[i] do
  begin
   if st='' then st:=ConCat(name,'=',AsString) else
                st:=ConCat(st,' ',name,'=',AsString);
  end;
  StrLCopy(trec.vals,pchar(st),sizeof(trec.vals));
 end;
end;

Procedure SetThing(thing:TJKThing;var trec:TThingRec);
var s,w:string;
    n,p:integer;
    v:TTPLValue;
begin
 With thing do
 begin
  {Mark:=trec.sec;}
  name:=trec.Name;
  X:=trec.X;
  Y:=trec.Y;
  Z:=trec.Z;
  PCH:=trec.PCH;
  YAW:=trec.YAW;
  ROL:=trec.ROL;
  layer:=trec.layer;

  s:=trec.Vals;
  p:=1;n:=0;
  while p<=length(s) do
  begin
   p:=GetWord(s,p,w);
   if n<Vals.count then v:=vals[n] else
   begin v:=TTPLValue.Create; Vals.Add(v); end;
   S2TPLVal(w,v);
   inc(n);
  end;

 end;
end;

Procedure StartUndoRec(name:string);
begin
end;

Procedure AddSecUndo(sec:TJKSector;how:integer);
begin
end;

Procedure AddThingUndo(th:TJKThing);
begin
end;

Procedure AddLightUndo(lt:TJedLight);
begin
end;

Procedure EndUndoRec;
begin
end;

Function GetRecName:string;
begin
end;

Procedure ApplyUndo;
begin
end;

end.
